__title__ = 'pydockenv'
__version__ = '0.3.1'
__author__ = 'Lou Marvin Caraig'
__author_email__ = 'loumarvincaraig@gmail.com'
__description__ = 'Python Virtualenv Powered by Docker'
__project_url__ = 'https://github.com/se7entyse7en/pydockenv'
__copyright__ = 'Copyright 2019 Lou Marvin Caraig'
